@@ -1,6 +1,7 @@
 #' Boxplots
 #'
 #' @name boxplot
+#' @keywords internal
 NULL
 
 #' Boxplot Function for Data Frame
@@ -12,12 +13,13 @@ NULL
 #' @param facets optional conditioning variables
 #' @param log whether to log transform numeric variable (auto-selected if NA)
 #' @param crit if log is NA, log-transform if mean/median ratio for non-missing values is greater than this value
-#' @param horizontal whether box/whisker axis should be horizontal (numeric x, categorical y); defaults TRUE if (var[[2]] is numeric
-#' @param scales passed to \code{\link[lattice]{xyplot}}; can be function(x = x, horizontal, log,...)
+#' @param horizontal whether box/whisker axis should be horizontal (numeric x, categorical y); defaults TRUE if var[[2]] is numeric
+#' @param scales passed to \code{\link[lattice]{xyplot}} (should be function(x = x, horizontal, log,...)) or \code{\link[ggplot2]{facet_grid}} or \code{\link[ggplot2]{facet_wrap}}
 #' @param panel panel function
-#' @param ref optional reference line(s) on numeric axis; can be function(x = x, var = con, ...)
+#' @param ref optional reference line(s) on numeric axis; can be function(x = x, var = con, ...) or NULL to suppress
 #' @param ref.col color for reference line(s)
 #' @param ref.lty line type for reference line(s)
+#' @param ref.lwd line size for reference line(s)
 #' @param ref.alpha transparency for reference line(s)
 #' @param nobs whether to include the number of observations under the category label
 #' @param na.rm whether to remove data points with one or more missing coordinates
@@ -25,11 +27,16 @@ NULL
 #' @param xlab x axis label
 #' @param numlab numeric axis label; can be function(x = x, var = numvar, log = ylog, ...)
 #' @param catlab categorical axis label; can be function(x = x, var = catvar, ...)
-#' @param aspect passed to \code{\link[lattice]{bwplot}}
+#' @param aspect passed to \code{\link[lattice]{bwplot}} or ggplot; use 'fill', NA, or NULL to calculate automatically
+#' @param as.table passed to \code{\link[lattice]{xyplot}}
 #' @param main character, or a function of x, yvar, xvar, facets, and log
 #' @param sub character, or a function of x, yvar, xvar, facets, and log
-#' @param par.settings default parameter settings
+#' @param settings default parameter settings: a list from which matching elements are passed to lattice (as par.settings) or  to ggplot theme()  and facet_wrap() or facet_grid().  \code{ncol} and \code{nrow} are used as layout indices for lattice (for homology with facet_wrap).
+#' @param padding numeric (will be recycled to length 4) giving plot margins in default units: top, right, bottom, left (in multiples of 5.5 points for ggplot)
 #' @param reverse if y is categorical, present levels in reverse order (first at top)
+#' @param pch special character for box median: passed to \code{\link[lattice]{panel.bwplot}}
+#' @param notch whether to draw notched boxes: passed to \code{\link[lattice]{panel.bwplot}}
+#' @param gg logical: whether to generate \code{ggplot} instead of \code{trellis}
 #' @param ... passed arguments
 #' @export
 #' @importFrom rlang quos UQ
@@ -51,32 +58,47 @@ boxplot_data_frame <- function(
   yvar,
   xvar,
   facets = NULL,
-  log = getOption('metaplot_log',FALSE),
-  crit = getOption('metaplot_crit',1.3),
-  horizontal = getOption('metaplot_horizontal',NULL),
-  scales = getOption('metaplot_boxplot_scales',NULL),
-  panel = getOption('metaplot_boxplot_panel',boxplot_panel),
-  ref = getOption('metaplot_ref',metaplot_ref),
-  ref.col = getOption('metaplot_ref.col','grey'),
-  ref.lty = getOption('metaplot_ref.lty','solid'),
-  ref.alpha = getOption('metaplot_ref.alpha',1),
-  nobs = getOption('metaplot_nobs',FALSE),
-  na.rm = getOption('metaplot_na.rm',TRUE),
+  log = metOption('metaplot_log_boxplot',FALSE),
+  crit = metOption('metaplot_crit_boxplot',1.3),
+  horizontal = metOption('metaplot_horizontal_boxplot',NULL),
+  scales = metOption('metaplot_scales_boxplot',NULL),
+  panel = metOption('metaplot_panel_boxplot',boxplot_panel),
+  ref = metOption('metaplot_ref_x_boxplot',metaplot_ref),
+  ref.col = metOption('metaplot_ref_col_boxplot','grey'),
+  ref.lty = metOption('metaplot_ref_lty_boxplot','solid'),
+  ref.lwd = metOption('metaplot_ref_lwd_boxplot',1),
+  ref.alpha = metOption('metaplot_ref_alpha_boxplot',1),
+  nobs = metOption('metaplot_nobs_boxplot',FALSE),
+  na.rm = metOption('metaplot_narm_boxplot',TRUE),
   xlab = NULL,
   ylab = NULL,
-  numlab = getOption('metaplot_lab',axislabel),
-  catlab = getOption('metaplot_lab',axislabel),
-  aspect = getOption('metaplot_aspect',1),
-  main = getOption('metaplot_main',NULL),
-  sub = getOption('metaplot_sub',NULL),
-  par.settings = standard.theme('pdf',color = FALSE),
-  reverse = getOption('metaplot_boxplot_reverse',TRUE),
+  numlab = metOption('metaplot_numlab_boxplot',axislabel),
+  catlab = metOption('metaplot_catlab_boxplot',axislabel),
+  aspect = metOption('metaplot_aspect_boxplot',1),
+  as.table = metOption('metaplot_astable_boxplot',TRUE),
+  main = metOption('metaplot_main_boxplot',NULL),
+  sub = metOption('metaplot_sub_boxplot',NULL),
+  settings = metOption('metaplot_settings_boxplot',NULL),
+  padding = metOption('metaplot_padding_boxplot', 1),
+  reverse = metOption('metaplot_reverse_boxplot',TRUE),
+  pch = metOption('metaplot_pch_boxplot','|'),
+  notch = metOption('metaplot_notch_boxplot',FALSE),
+  gg = metOption('metaplot_gg_boxplot',FALSE),
   ...
 ){
+  settings <- as.list(settings)
+  if(is.null(names(settings))) names(settings) <- character(0)
+  aspect <- metaplot_aspect(aspect, gg)
   stopifnot(inherits(x, 'data.frame'))
   stopifnot(is.character(xvar))
   stopifnot(is.character(yvar))
   if(!is.null(facets))stopifnot(is.character(facets))
+  stopifnot(is.numeric(padding))
+  padding <- rep(padding, length.out = 4)
+  par.settings <- settings[names(settings) %in% names(trellis.par.get())]
+  par.settings <- parintegrate(par.settings, padding)
+  if(gg)padding <- unit(padding * 5.5, 'pt')
+
   y <- x
   stopifnot(all(c(xvar,yvar,facets) %in% names(y)))
 
@@ -122,11 +144,15 @@ boxplot_data_frame <- function(
       log <- mean(y[[con]],na.rm = TRUE)/median(y[[con]],na.rm = TRUE) > crit
     }
   }
-  if(log)if(any(y[[con]] <= 0, na.rm = TRUE)){
-    warning(con, ' must be positive for log scale')
-    log <- FALSE
+
+  bad <- !is.na(y[[con]]) & y[[con]] <= 0
+  bad[is.na(bad)] <- FALSE
+  if(log && any(bad)){
+    warning('dropping ',sum(bad), ' non-positive records for log scale')
+    y <- y[!bad,]
   }
-  if(log){
+
+  if(log & !gg){
     ref <- ref[ref > 0]
     if(length(ref)) ref <- log10(ref)
     if(!length(ref)) ref <- NULL
@@ -153,25 +179,87 @@ boxplot_data_frame <- function(
     num <- sapply(lev,function(l)sum(na.rm = TRUE, y[[cat]] == l))
     levels(y[[cat]]) <- paste(lev,num,sep = '\n')
   }
+  if(is.null(scales) && gg) scales <- 'fixed'
   if(is.null(scales)) scales <- function(x = y, horizontal = horizontal, log = log, ...){
     s <- list(
       tck = c(1,0),
+      alternating = FALSE,
       x = list(log = log,equispaced.log = FALSE)
     )
     if(!horizontal)s <- list(
       tck = c(1,0),
+      alternating = FALSE,
       y = list(log = log,equispaced.log = FALSE)
     )
     s
   }
-  if(is.character(scales)) scales <- match.fun(scales)
-  if(!is.function(scales)) stop('scales must be NULL, or function(x, horizontal, log,...), or the name of a function')
-  scales <- scales(x=x, horizontal = horizontal, log = log, ...)
+  if(is.character(scales) && !gg) scales <- match.fun(scales)
+  if(!is.function(scales) && !gg) stop('scales must be NULL, or function(x, horizontal, log,...), or the name of a function')
+  if(!gg) scales <- scales(x=x, horizontal = horizontal, log = log, ...)
 
   if(!is.null(main))if(is.function(main)) main <- main(x = x, yvar = yvar, xvar = xvar, facets = facets, log = log, ...)
   if(!is.null(sub))if(is.function(sub)) sub <- sub(x = x, yvar = yvar, xvar = xvar, facets = facets, log = log, ...)
 
-  bwplot(
+  if(gg){
+    plot <- ggplot(data = y, aes_string(cat, con))
+    plot <- plot  +
+      geom_boxplot(notch = notch) +
+      xlab(catlab) +
+      ylab(numlab) +
+      ggtitle(main, subtitle = sub)
+
+    # scale aesthetics
+    panels <- nrow(unique(y[facets]))
+    if(!panels) panels <- 1
+
+    ref.col <- rep(ref.col, length.out = length(ref))
+    ref.lty <- rep(ref.lty, length.out = length(ref))
+    ref.lwd <- rep(ref.lwd, length.out = length(ref))
+    ref.alpha <- rep(ref.alpha, length.out = length(ref))
+
+    ref.col <- rep(ref.col, times = panels)
+    ref.lty <- rep(ref.lty, times = panels)
+    ref.lwd <- rep(ref.lwd, times = panels)
+    ref.alpha <- rep(ref.alpha, times = panels)
+
+    if(length(ref)) plot <- plot +
+      geom_hline(
+        yintercept = ref,
+        color = ref.col,
+        linetype = ref.lty,
+        size = ref.lwd,
+        alpha = ref.alpha
+      )
+    theme_settings <- list(aspect.ratio = aspect, plot.margin = padding)
+    theme_extra <- settings[names(settings) %in% names(formals(theme))]
+    theme_settings <- merge(theme_settings, theme_extra)
+    plot <- plot + do.call(theme, theme_settings)
+
+    if(log) plot <- plot + scale_y_continuous(
+      trans = log_trans(),
+      breaks = base_breaks()
+    )
+    facet_args <- list()
+    if(length(facets) ==1) facet_args[[1]] <- facets[[1]]
+    if(length(facets) > 1) facet_args[[1]] <- as.formula(
+          paste(
+            sep='~',
+            facets[[1]],
+            facets[[2]]
+          )
+        )
+    facet_args$scales <- scales
+    facet_extra <- list()
+    if(length(facets) == 1) facet_extra <- settings[names(settings) %in% names(formals(facet_wrap))]
+    if(length(facets) >  1) facet_extra <- settings[names(settings) %in% names(formals(facet_grid))]
+    facet_args <- merge(facet_args, facet_extra)
+    if(length(facets) == 1) plot <- plot + do.call(facet_wrap, facet_args)
+    if(length(facets) >  1) plot <- plot + do.call(facet_grid, facet_args)
+    if(horizontal) plot <- plot + coord_flip()
+    return(plot)
+  }
+
+  args <- list(
     formula,
     data = y,
     aspect = aspect,
@@ -180,15 +268,25 @@ boxplot_data_frame <- function(
     scales = scales,
     ylab = ylab,
     xlab = xlab,
+    as.table = as.table,
     main = main,
     sub = sub,
     panel = panel,
     ref = ref,
     ref.col = ref.col,
     ref.lty = ref.lty,
+    ref.lwd = ref.lwd,
     ref.alpha = ref.alpha,
-    ...
+    pch = pch,
+    notch = notch
   )
+
+  args <- c(args, list(...))
+  if(all(c('ncol','nrow') %in% names(settings))){
+    layout <- c(settings$ncol, settings$nrow)
+    args <- c(args, list(layout = layout))
+  }
+  do.call(bwplot, args)
 }
 
 #' Panel Function for Metaplot Boxplot
@@ -204,15 +302,16 @@ boxplot_data_frame <- function(
 #' @param notch passed to \code{\link[lattice]{panel.bwplot}}
 #' @param ref.col passed to \code{\link[lattice]{panel.abline}} as col
 #' @param ref.lty passed to \code{\link[lattice]{panel.abline}} as lty
+#' @param ref.lwd passed to \code{\link[lattice]{panel.abline}} as lwd
 #' @param ref.alpha passed to \code{\link[lattice]{panel.abline}} as alpha
 #'
-boxplot_panel <- function(ref = NULL, horizontal,pch = '|',notch=FALSE, ref.col, ref.lty, ref.alpha, ...){
+boxplot_panel <- function(ref = NULL, horizontal,pch = '|',notch=FALSE, ref.col, ref.lty, ref.lwd, ref.alpha, ...){
   panel.bwplot(horizontal = horizontal, pch = pch, notch = notch, ...)
   if(length(ref)){
     if(horizontal) {
-      panel.abline(v = ref, col = ref.col, lty = ref.lty, alpha = ref.alpha)
+      panel.abline(v = ref, col = ref.col, lty = ref.lty, lwd = ref.lwd, alpha = ref.alpha)
     }else{
-      panel.abline(h = ref, col = ref.col, lty = ref.lty, alpha = ref.alpha)
+      panel.abline(h = ref, col = ref.col, lty = ref.lty, lwd = ref.lwd, alpha = ref.alpha)
     }
   }
 }
@@ -223,6 +322,7 @@ boxplot_panel <- function(ref = NULL, horizontal,pch = '|',notch=FALSE, ref.col,
 #' @param x data.frame
 #' @param ... passed to fun
 #' @param fun function that does the actual plotting
+#' @param verbose generate messages describing process
 #' @export
 #' @importFrom rlang f_rhs
 #' @family mixedvariate plots
@@ -234,15 +334,25 @@ boxplot_panel <- function(ref = NULL, horizontal,pch = '|',notch=FALSE, ref.col,
 #' Theoph %<>% mutate(site = ifelse(as.numeric(Subject) > 6, 'Site A','Site B'))
 #' boxplot(Theoph,'Subject','conc')
 #' boxplot(Theoph,Subject,conc)
+#' boxplot(Theoph,Subject,conc, gg = T)
 #' boxplot(Theoph,conc,Subject)
+#' boxplot(Theoph,conc,Subject, gg = T)
 #' boxplot(Theoph,conc,Subject,site)
+#' boxplot(Theoph,conc,Subject,site, gg = T)
+#' boxplot(Theoph,conc,Subject,site, gg = T, scales = 'free_x')
 #' attr(Theoph,'title') <- 'Theophylline'
 #' boxplot(Theoph, Subject, conc, main = function(x,...)attr(x,'title'))
+#' boxplot(Theoph, Subject, conc, main = function(x,...)attr(x,'title'), gg = T)
 #' boxplot(Theoph, Subject, conc, sub= function(x,...)attr(x,'title'))
+#' boxplot(Theoph, Subject, conc, sub= function(x,...)attr(x,'title'), gg = T)
+#' boxplot(Theoph %>% filter(conc > 0),Subject,conc, log = T)
+#' boxplot(Theoph %>% filter(conc > 0),Subject,conc, log = T, gg = T)
+
 boxplot.data.frame <- function(
   x,
   ...,
-  fun = getOption('metaplot_box','boxplot_data_frame')
+  fun = getOption('metaplot_box','boxplot_data_frame'),
+  verbose = metOption('metaplot_verbose_boxplot',FALSE)
 ){
   args <- quos(...)
   args <- lapply(args,f_rhs)
@@ -263,6 +373,9 @@ boxplot.data.frame <- function(
     facets = facets
   )
   args <- c(formal, other)
+  if(verbose){
+    if(is.character(fun))message('calling ', fun) else message('calling fun')
+  }
   do.call(fun, args)
 }
 
